#include "xstatus.h"
#include "lwip/sockets.h"
#include "xil_printf.h"
#include "string.h"
#include "http_server.h"
#include "webpage.h"

int generate_http_response(int sd, HttpRequest_t *Request)
{
    switch (Request->Method) {
    case HTTP_GET:
        return respond_http_get(sd, Request);
    case HTTP_POST:
        return respond_http_post(sd, Request);
    case HTTP_OPTIONS:
        return respond_http_options(sd);
    default:
        return do_http_501(sd);
    }
}

int respond_http_options(int sd)
{
    char *Response = "HTTP/1.1 204 No Content\r\n"
                     "Access-Control-Allow-Origin: null\r\n"
                     "Access-Control-Allow-Methods: POST, GET, OPTIONS\r\n"
                     "Access-Control-Allow-Headers: Content-Type\r\n"
                     "Accept-Encoding: *\r\n"
                     "Accept-Language: *\r\n"
                     "Connection: close\r\n"
                     "\r\n"
                     "\0";

    if (write(sd, Response, strlen(Response)) < 0) {
        xil_printf("Error writing http OPTIONS response to socket.\r\n");
        return XST_FAILURE;
    }
    return XST_SUCCESS;
}

int respond_http_get(int sd, HttpRequest_t *Request)
{
    int Status;
    char MessageBuf[256];

    switch (Request->Command) {
    case EVENT_STREAM_URL:
        strcpy(MessageBuf, "data: [Web Server] 200 OK: SSE connection "
                "opened.\n\n");
        return do_http_200(sd, MessageBuf, "sse");
        break;
    default:
        return do_http_200(sd, webpage, "htm");
    }
}

int respond_http_post(int sd, HttpRequest_t *Request)
{
    int Status;
    char MessageBuf[256];

    switch (Request->Command) {
    case UPDATE_MULTIBOOT_CMD:
        sprintf(MessageBuf, "[Web Server] 200 OK: Received boot image of size %d bytes. "
                "One of multiboot firmware sectors in flash will be updated.",
                Request->ContentLength);
        Status = do_http_200(sd, MessageBuf, "txt");
        break;
    case UPDATE_GOLDEN_CMD:
        sprintf(MessageBuf, "[Web Server] 200 OK: Received boot image of size %d bytes. "
                "Golden firmware sector in flash will be updated.",
                Request->ContentLength);
        Status = do_http_200(sd, MessageBuf, "txt");
        break;
    case UNKNOWN_CMD:
        Status = do_http_400(sd);
        break;
    default:
        Status = do_http_400(sd);
        break;
    }

    return Status;
}

int do_http_200(int sd, const char *Message, char* MessageType)
{
    char HeaderBuf[1024];
    char lbuf[40];

    /* Format the beginning of the response header */
    strcpy(HeaderBuf, "HTTP/1.1 200 OK\r\n"
                      "Access-Control-Allow-Origin: null\r\n"
                      "Cache-Control: no-cache\r\n"
                      "Content-Type: ");

    /* Format the Content-Type header attribute */
    if (!strncmp(MessageType, "htm", 3))
        strcat(HeaderBuf, "text/html\r\n");  /* html */
    else if (!strncmp(MessageType, "sse", 3))
        strcat(HeaderBuf, "text/event-stream\r\n");  /* HTML5-SSE */
    else
        strcat(HeaderBuf, "text/plain\r\n"); /* for other types */

    /* Format the Content-Length header attribute */
    sprintf(lbuf, "Content-length: %d\r\n", strlen(Message));
    strcat(HeaderBuf, lbuf);

    /* Format the Connection header attribute */
    // strcat(HeaderBuf, "Connection: Keep-Alive\r\n");
    strcat(HeaderBuf, "Connection: close\r\n");
    strcat(HeaderBuf, "\r\n");

    /* Send the response header */
    if (write(sd, HeaderBuf, strlen(HeaderBuf)) < 0) {
        xil_printf("error writing http 200 response to socket\r\n");
        return XST_FAILURE;
    }

    /* Send the response content */
    if (write(sd, Message, strlen(Message)) < 0) {
        xil_printf("error writing http 200 response to socket\r\n");
        return XST_FAILURE;
    }
    return XST_SUCCESS;
}

int do_http_501(int sd)
{
    char *Response = "HTTP/1.1 501 Not Implemented\r\n"
                     "Access-Control-Allow-Origin: null\r\n"
                     "Access-Control-Allow-Methods: POST, GET, OPTIONS\r\n"
                     "Access-Control-Allow-Headers: Content-Type\r\n"
                     "Accept-Encoding: *\r\n"
                     "Accept-Language: *\r\n"
                     "Cache-Control: no-cache\r\n"
                     "Connection: close\r\n"
                     "\r\n"
                     "\0";

    if (write(sd, Response, strlen(Response)) < 0) {
        xil_printf("Error writing http 501 response to socket.\r\n");
        return XST_FAILURE;
    }
    return XST_SUCCESS;
}

int do_http_400(int sd)
{
    const char *Response = "HTTP/1.1 400 Bad Request\r\n"
                     "Access-Control-Allow-Origin: null\r\n"
                     "Access-Control-Allow-Methods: POST, GET, OPTIONS\r\n"
                     "Access-Control-Allow-Headers: Content-Type\r\n"
                     "Accept-Encoding: *\r\n"
                     "Accept-Language: *\r\n"
                     "Content-Length: 90\r\n"
                     "Cache-Control: no-cache\r\n"
                     "Connection: close\r\n"
                     "\r\n"
                     "[Web Server] 400 Bad Request: No or unsupported command received. No "
                     "action will be taken.";

    if (write(sd, Response, strlen(Response)) < 0) {
        xil_printf("Error writing http 400 response to socket.\r\n");
        return XST_FAILURE;
    }
    return XST_SUCCESS;
}

