#include "xil_printf.h"
#include "xil_types.h"
#include "string.h"
#include "http_server.h"
#include "xstatus.h"

int http_port = 80;

void print_http_server_header()
{
    xil_printf("%20s %6d %s\r\n", "http server",
                        http_port,
                        "http://<board_ip> 80");
}

u8 *GetHttpDataStart(u8 *RecvData)
{
    u8 *HttpPayloadStart;

    HttpPayloadStart = (u8 *)strstr((const char *)RecvData, "\r\n\r\n");
    if (HttpPayloadStart != NULL) {
        return HttpPayloadStart + 4;
    } else {
        return NULL;
    }
}

void GetRequestMethodAndCommand(const u8 *HttpHeader, HttpRequest_t *Request)
{
    /* Supported methods definition */
    const char *method_1 = "GET";
    const char *method_2 = "POST";
    const char *method_3 = "OPTIONS";

    /* Supported commands definition */
    const char *cmd_1 = "cmd/update-golden";
    const char *cmd_2 = "cmd/update-multiboot";
    const char *cmd_3 = "stream";
    char *CommandStart;

    if (!strncmp((const char *)HttpHeader, method_1, strlen(method_1))) {
        Request->Method = HTTP_GET;
        /* skip past 'GET /' */
        CommandStart = (char *)HttpHeader + 5;
        if (!strncmp(CommandStart, cmd_3, strlen(cmd_3)))
            Request->Command = EVENT_STREAM_URL;
        else
            /* In case of no or unsupported command */
            Request->Command = UNKNOWN_CMD;

    } else if (!strncmp((const char *)HttpHeader, method_2, strlen(method_2))){
        Request->Method = HTTP_POST;
        /* skip past 'POST /' */
        CommandStart = (char *)HttpHeader + 6;
        if (!strncmp(CommandStart, cmd_1, strlen(cmd_1)))
            Request->Command = UPDATE_GOLDEN_CMD;
        else if (!strncmp(CommandStart, cmd_2, strlen(cmd_2)))
            Request->Command = UPDATE_MULTIBOOT_CMD;
        else
            /* In case of no or unsupported command */
            Request->Command = UNKNOWN_CMD;

    } else if (!strncmp((const char *)HttpHeader, method_3, strlen(method_3))){
        Request->Method = HTTP_OPTIONS;
        Request->Command = UNKNOWN_CMD;
    } else {
        Request->Method = HTTP_UNKNOWN;
        Request->Command = UNKNOWN_CMD;
    }
}

u32 GetContentLength(const u8 *HttpHeader)
{
    char *ContentLengthPtr;

    ContentLengthPtr = strstr((const char *)HttpHeader, "Content-Length");
    if (ContentLengthPtr == NULL) {
        return 0;
    } else {
        return (u32)(strtol(ContentLengthPtr + 16, NULL, 10));
    }
}

void ProcessRequestHeader(const u8 *HttpHeader, HttpRequest_t *Request)
{
    GetRequestMethodAndCommand(HttpHeader, Request);
    Request->ContentLength = GetContentLength(HttpHeader);
}
