/***************************** Include Files *********************************/
#include <stdio.h>
#include "xparameters.h"
#include "netif/xadapter.h"
#include "platform_config.h"
#include "xil_printf.h"
#include "lwip/inet.h"
#include "qspi.h"
#include "xstatus.h"
/************************** Constant Definitions *****************************/
#define THREAD_STACKSIZE 1024
/**************************** Type Definitions *******************************/

/************************** Function Prototypes ******************************/
int main_thread();
void http_server_thread();
void lwip_init();
/************************** Variable Definitions *****************************/
static struct netif server_netif;
struct netif *echo_netif;

void print_ip(char *msg, ip_addr_t *ip)
{
	xil_printf(msg);
	xil_printf("%d.%d.%d.%d\n\r", ip4_addr1(ip), ip4_addr2(ip),
			ip4_addr3(ip), ip4_addr4(ip));
}

void print_ip_settings(ip_addr_t *ip, ip_addr_t *mask, ip_addr_t *gw)
{

	print_ip("Board IP: ", ip);
	print_ip("Netmask : ", mask);
	print_ip("Gateway : ", gw);
}

int main()
{
	sys_thread_new("main_thrd", (void(*)(void*))main_thread, 0,
	                THREAD_STACKSIZE,
	                DEFAULT_THREAD_PRIO);
	vTaskStartScheduler();
	xil_printf("Main Thread Created, Scheduler Starter");
	while(1);
	return 0;
}

void network_thread(void *p)
{
    struct netif *netif;

    /* the mac address of the board. this should be unique per board */
    unsigned char mac_ethernet_address[] = { 0x00, 0x0a, 0x35, 0x00, 0x01, 0x02 };

    ip_addr_t ipaddr, netmask, gw;


    netif = &server_netif;

    // xil_printf("\r\n\r\n");
    // xil_printf("-----http server------\r\n");

    /* initliaze IP addresses to be used */
    IP4_ADDR(&ipaddr,  192, 168, 1, 10);
    IP4_ADDR(&netmask, 255, 255, 255,  0);
    IP4_ADDR(&gw,      192, 168, 1, 1);

    /* print out IP settings of the board */

    // print_ip_settings(&ipaddr, &netmask, &gw);

    /* Add network interface to the netif_list, and set it as default */
    if (!xemac_add(netif, &ipaddr, &netmask, &gw,
    		mac_ethernet_address, PLATFORM_EMAC_BASEADDR)) {
    	xil_printf("Error adding N/W interface\r\n");
	return;
    }

    netif_set_default(netif);

    /* specify that the network if is up */
    netif_set_up(netif);

    /* start packet receive thread - required for lwIP operation */
    sys_thread_new("xemacif_input_thread", (void(*)(void*))xemacif_input_thread, netif,
            THREAD_STACKSIZE,
            DEFAULT_THREAD_PRIO);

    /*
    xil_printf("\r\n");
    xil_printf("%20s %6s %s\r\n", "Server", "Port", "Connect With..");
    xil_printf("%20s %6s %s\r\n", "--------------------", "------", "--------------------");

    print_http_server_header();
    xil_printf("\r\n"); */
    sys_thread_new("http_server", http_server_thread, 0,
		THREAD_STACKSIZE,
		DEFAULT_THREAD_PRIO);
    vTaskDelete(NULL);

    return;
}

int main_thread()
{
	int Status, i;

	/* initialize lwIP before calling sys_thread_new */
    lwip_init();

    /* initialize QSPI Flash */
	Status =  FlashInit();
	if (Status != XST_SUCCESS) {
		xil_printf("Failed to Initialize QSPI Flash");
	}

    /* any thread using lwIP should be created using sys_thread_new */
    sys_thread_new("NW_THRD", network_thread, NULL,
		THREAD_STACKSIZE, DEFAULT_THREAD_PRIO);

    vTaskDelete(NULL);
    return 0;
}
