/************************** Constant Definitions *****************************/
#define PAGE_COUNT 8
#define MAX_PAGE_SIZE 256

/*
 * The following constants define the offsets within a FlashBuffer data
 * type for each kind of data.  Note that the read data offset is not the
 * same as the write data because the QSPI driver is designed to allow full
 * duplex transfers such that the number of bytes received is the number
 * sent and received.
 */
#define COMMAND_OFFSET		0 /* Flash instruction */
#define ADDRESS_1_OFFSET	1 /* MSB byte of address to read or write */
#define ADDRESS_2_OFFSET	2 /* Middle byte of address to read or write */
#define ADDRESS_3_OFFSET	3 /* LSB byte of address to read or write */
#define DATA_OFFSET		    4 /* Start of Data for Read/Write */
#define DUMMY_OFFSET        4 /* Dummy byte offset for fast, dual and quad
                              /* reads */
#define DUMMY_SIZE		    1 /* Number of dummy bytes for fast, dual and
                              /* quad reads */
#define RD_ID_SIZE		    4 /* Read ID command + 3 bytes ID response */
#define BULK_ERASE_SIZE		1 /* Bulk Erase command size */
#define SEC_ERASE_SIZE		4 /* Sector Erase command + Sector address */
#define BANK_SEL_SIZE	    2 /* BRWR or EARWR command + 1 byte bank value */
#define RD_CFG_SIZE		    2 /* 1 byte Configuration register + RD CFG command*/
#define WR_CFG_SIZE		    3 /* WRR command + 1 byte each Status and Config Reg*/
#define DIE_ERASE_SIZE	    4	/* Die Erase command + Die address */
/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

int FlashInit(void);

void FlashErase(u32 Address, u32 ByteCount);

void FlashWrite(u32 FlashAddr, const u8 *SourceAddr, u32 ByteCount);

void FlashRead(u8 *DestAddr, u32 FlashAddr, u32 ByteCount);

void FlashWriteWord(u32 FlashAddr, const u32 Word);

void BulkErase(void);

/************************** Variable Definitions *****************************/
